
# https://bugzilla.redhat.com/show_bug.cgi?id=995136#c12
%global _find_debuginfo_dwz_opts %{nil}
%global _dwz_low_mem_die_limit 0

#I really need this, otherwise "version_ldflags=$(kube::version_ldflags)"
# does not work
%global _buildshell /bin/bash
%global _checkshell /bin/bash

# k8s release version major.minor
%global major          1
%global minor          17
%global patch          9
%global image_registry container-registry.oracle.com/olcne
%global k8s_repo       kubernetes

# GOLANG version used to override k8s go version in build
%global golang_version 1.13.4

Name:          kubernetes
Version:       1.17.9
Release:       1.0.1%{?dist}
Summary:       Container cluster management
License:       ASL 2.0
Group:         System/Management
URL:           https://kubernetes.io
Vendor:        Oracle America
ExclusiveArch: x86_64 ppc64le %{arm} aarch64
Source:        kubernetes-%{version}.tar.bz2
Source31:      genmanpages.sh

Source40:      10-kubeadm.conf
Source41:      kubelet.service
Source43:      k8s.conf
Source46:      kubectl-proxy.service
Source47:      10-kubectl-proxy.conf
Source51:      kubelet

%description
%{summary}
# Setup golang dependency
BuildRequires: golang >= %{golang_version}

BuildRequires: systemd
BuildRequires: rsync
BuildRequires: go-md2man
BuildRequires: git

%package -n kubelet
Summary: Container cluster management
BuildRequires: golang >= %{golang_version}
BuildRequires: systemd
BuildRequires: curl
BuildRequires: go-md2man
Requires: iptables >= 1.4.21
Requires: socat
Requires: util-linux
Requires: ethtool
Requires: iproute
Requires: iproute-tc
Requires: ebtables
Requires: conntrack
Requires: kubernetes-cni >= 0.7.1
Requires: openssl
Requires: kata = 1.7.3-1.0.7.el7
Requires: %{_sysconfdir}/crio/1.17

%description -n kubelet
The node agent of Kubernetes, the container cluster manager.

%package -n kubectl
Summary: Command-line utility for interacting with a Kubernetes cluster.

%description -n kubectl
Command-line utility for interacting with a Kubernetes cluster.

%package -n kubeadm
Summary: Command-line utility for administering a Kubernetes cluster.

%description -n kubeadm
Command-line utility for administering a Kubernetes cluster.

%package -n kubeadm-container-images
Summary: Contains Oracle built k8s docker images

%description -n kubeadm-container-images
Contains Oracle built k8s docker images

%prep
%setup -q -n %{k8s_repo}-%{version}

dirs=$(ls | grep -v "^Godeps")

# Move all the code under src/k8s.io/kubernetes directory
mkdir -p src/k8s.io/kubernetes
mv $(ls | grep -v "^src$") src/k8s.io/kubernetes/.

%build
pushd src/k8s.io/kubernetes/
export KUBE_EXTRA_GOPATH=$(pwd)/Godeps/_workspace

export GOPATH=$(pwd)/Godeps/_workspace
export KUBE_GIT_TREE_STATE=clean
export KUBE_GIT_VERSION=v%{version}+%{release}
export KUBE_GIT_MAJOR=%{major}
export KUBE_GIT_MINOR=%{minor}

go version

make WHAT='cmd/kube-proxy cmd/kube-apiserver cmd/kube-controller-manager cmd/kubelet cmd/kube-scheduler cmd/kubectl cmd/kubeadm'

chmod +x build-k8s-docker.sh
./build-k8s-docker.sh %{version} _output/bin/ %{golang_version} %{image_registry}

bash hack/update-generated-docs.sh

# convert md to man
pushd docs
pushd admin
cp kube-apiserver.md kube-controller-manager.md kube-proxy.md kube-scheduler.md kubelet.md ..
popd
cp %{SOURCE31} genmanpages.sh
bash genmanpages.sh
popd
popd

%install
pushd src/k8s.io/kubernetes/
. hack/lib/init.sh

%ifarch ppc64le
output_path="${KUBE_OUTPUT_BINPATH}"
%else
output_path="${KUBE_OUTPUT_BINPATH}/linux/amd64"
%endif

binaries=(kubelet kubectl kubeadm)
install -m 755 -d %{buildroot}%{_bindir}
for bin in "${binaries[@]}"; do
  echo "+++ INSTALLING ${bin}"
  install -p -m 755 -t %{buildroot}%{_bindir} ${output_path}/${bin}
done

# install the bash completion
install -d -m 0755 %{buildroot}%{_datadir}/bash-completion/completions/
%{buildroot}%{_bindir}/kubectl completion bash > %{buildroot}%{_datadir}/bash-completion/completions/kubectl

# install manpages
install -d %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubectl* %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubelet* %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubeadm* %{buildroot}%{_mandir}/man1
# from k8s tarball copied docs/man/man1/*.1

# EXTRA FOR KUBEADM/KUBELET
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d/
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d %{SOURCE40}
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d %{SOURCE47}

install -m 755 -d %{buildroot}%{_sysconfdir}/kubernetes/manifests/
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/
install -m 755 -d %{buildroot}/usr/libexec/kubernetes/kubelet-plugins/volume/exec/

install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE41}
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE46}
install -d -m 755 %{buildroot}/etc/sysctl.d
install -p -m 644 -t %{buildroot}/etc/sysctl.d %{SOURCE43}

install -d -m 755 %{buildroot}/etc/sysconfig
install -p -m 755 -t %{buildroot}/etc/sysconfig %{SOURCE51}

# EXTRA FOR DOCKER IMAGES
mkdir -p %{buildroot}/usr/local/share/kubeadm
mkdir -p %{buildroot}/usr/local/share/olcne
install -m 755 -d %{buildroot}/usr/local/share/kubeadm
install -m 755 -d %{buildroot}/usr/local/share/olcne
images=(kube-apiserver.tar kube-controller-manager.tar kube-scheduler.tar kube-proxy.tar pause.tar)
for bin in "${images[@]}"; do
  echo "+++ INSTALLING DOCKER IMAGES ${bin}"
  install -p -m 755 -t %{buildroot}/usr/local/share/olcne ${output_path}/oracle_docker/${bin}
done

popd
mv src/k8s.io/kubernetes/*.md .
mv src/k8s.io/kubernetes/LICENSE .
mv src/k8s.io/kubernetes/THIRD_PARTY_LICENSES.txt .

%check
#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files -n kubelet
%license LICENSE THIRD_PARTY_LICENSES.txt
%{_bindir}/kubelet
%{_sysconfdir}/sysconfig/kubelet
%{_sysconfdir}/systemd/system/kubelet.service
%{_sysconfdir}/systemd/system/kubelet.service.d/10-kubeadm.conf
%{_sysconfdir}/kubernetes/manifests/
/usr/libexec/kubernetes/kubelet-plugins/volume/exec
%doc *.md
%{_mandir}/man1/kubelet*

%files -n kubeadm
%license LICENSE THIRD_PARTY_LICENSES.txt
%doc *.md
%{_mandir}/man1/kubeadm*
%{_bindir}/kubeadm
/etc/sysctl.d/k8s.conf

%files -n kubectl
%license LICENSE THIRD_PARTY_LICENSES.txt
%{_bindir}/kubectl
%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/10-kubectl-proxy.conf
%{_sysconfdir}/systemd/system/kubectl-proxy.service
%doc *.md
%{_mandir}/man1/kubectl*
%{_datadir}/bash-completion/completions/kubectl

%files -n kubeadm-container-images
%license LICENSE THIRD_PARTY_LICENSES.txt
/usr/local/share/olcne/kube-apiserver.tar
/usr/local/share/olcne/kube-controller-manager.tar
/usr/local/share/olcne/kube-scheduler.tar
/usr/local/share/olcne/kube-proxy.tar
/usr/local/share/olcne/pause.tar

%pre -n kubeadm
# check if this is an upgrade
if [ "$1" == "2" ]; then
    #check if we are upgrading from earlier than 1.11
    # upgrade
    RPM_VERSION=`/bin/rpm -q --queryformat='%{VERSION}' kubeadm 2>&1`
    _major_version=`echo $RPM_VERSION | /bin/awk -F '.' '{print $2}'`
    # any version prior to 1.11 are not compatible
    if [ "${_major_version}" -lt "11" ]; then
        if [ -z "${FORCE_UPDATE_12_FROM_9}" ]; then
            echo 'Can not upgrade kubeadm from version prior to 1.11 due to upgrade compatibility'
            exit 1
        fi
        echo "Upgrading kubeadm forcefully from version earlier that 1.11"
    fi
fi

%post -n kubeadm
sysctl -p /etc/sysctl.d/k8s.conf

%post -n kubelet
%systemd_post kubelet
if [ $1 -gt 1 ] ; then
  systemctl daemon-reload >/dev/null 2>&1
  systemctl try-restart kubelet >/dev/null 2>&1
fi

%preun -n kubelet
%systemd_preun kubelet

%postun -n kubelet
%systemd_postun_with_restart kubelet

%changelog
* Wed Jul 15 2020 Olcne-Builder Jenkins <olcne-builder_us@oracle.com> - 1.17.9-%{release}
- Added Oracle specific build files for Kubernetes
